// A linked list with each node containing two integers.
struct N {
  int a;
  int b;
  struct N * next;
};

int main()
{
  struct N *ptr = malloc(sizeof(N));
  N->next = NULL;
  (*N).a = 1;
  (*N).b = 2;
  return 0;
}
