""" Type Classes.
    
    Used to structure the Type Heirarchy.
    
    This file is AUTO GENERATED from
      _types_cfg.yaml using _ast_gen.py
    The header is stored in _types_header.py
"""

import node

class Type(node.Node):
  """ The Base Type class."""
  pass


