# Test Expressions
2
3 + 5
2 + 3 * 5
a + b * c
a << 2
(a - 2) * (b + c) /d
a % b
(c >> 3) * d
(a | b) ^ (c & d)
(a and b) or (c and d)
(a < b) and (c <= d) or (e > f) and (g > h)
(a == b)
(a != b)
not (a == b)
(-a)
(+a)
a < b < c
a <= b > c
a in x
a not in y
a is None
b is not None
(2, 3)
[5, 6]
"Redhawk"
"Redhawk" != x
a.x
b.y.z
x if condition else y
[2*x for x in li if x %2 == 0 if x%3 == 0]
(g for g in it)
(3*g for g in [1, 2, 3, 4])
lambda x: 2*x
lambda (x, y): x + y
lambda x, *y: y
li[0]
li[-1]
li[1:3]
li[:]
li[1:3:5]
f()
{2: '3', 'a': b}
2**3
Repr(2+3)
